LOG_LOCATION = '/opt/log/road_collisions/road_collisions.log'


WEEKDAY_MAP = {
    1: 'sunday',
    2: 'monday',
    3: 'tuesday',
    4: 'wednesday',
    5: 'thursday',
    6: 'friday',
    7: 'saturday'
}

GENDER_MAP = {
    'm': 'male',
    'f': 'female'
}

VEHICLE_TYPE_MAP = {
    1: 'bicycle',
    2: 'motorcycle',
    3: 'car',
    4: 'goods_vehicle',
    5: 'bus',
    6: 'other'
}

HOUR_MAP = {
    1: '7-10',
    2: '10-16',
    3: '16-19',
    4: '19-23',
    5: '23-3',
    6: '3-7'
}

CIRCUMSTANCS_MAP = {
    1: 'pedestrian',
    2: 'single_vehicle_only',
    3: 'head_on_conflict',
    4: 'head_on_right_turn',
    5: 'angle_both_straight',
    6: 'angle_right_turn',
    7: 'rear_end_straight',
    8: 'rear_end_right_turn',
    9: 'rear_end_left_turn',
    10: 'other'
}

SEVERITY_MAP = {
    0: 'all',
    1: 'fatal',
    2: 'serious',
    3: 'minor'
}

COUNTY_MAP = {
    1: 'carlow',
    2: 'cavan',
    3: 'clare',
    4: 'cork',
    5: 'donegal',
    6: 'dublin',
    7: 'galway',
    8: 'kerry',
    9: 'kildare',
    10: 'kilkenny',
    11: 'laois',
    12: 'leitrim',
    13: 'limerick',
    14: 'longford',
    15: 'louth',
    16: 'mayo',
    17: 'meath',
    18: 'monaghan',
    19: 'offaly',
    20: 'roscommon',
    21: 'sligo',
    22: 'Tipperary',
    23: 'Waterford',
    24: 'Westmeath',
    25: 'wexford',
    26: 'wicklow'
}
